#!/usr/bin/env python

from .acfun import *
from .alive import *
from .baidu import *
from .bilibili import *
from .blip import *
from .catfun import *
from .cbs import *
from .cntv import *
from .coursera import *
from .dailymotion import *
from .douban import *
from .ehow import *
from .facebook import *
from .freesound import *
from .google import *
from .ifeng import *
from .instagram import *
from .iqiyi import *
from .joy import *
from .jpopsuki import *
from .ku6 import *
from .kugou import *
from .kuwo import *
from .letv import *
from .magisto import *
from .miomio import *
from .mixcloud import *
from .mtv81 import *
from .netease import *
from .nicovideo import *
from .pptv import *
from .qq import *
from .sina import *
from .sohu import *
from .songtaste import *
from .soundcloud import *
from .theplatform import *
from .tudou import *
from .tumblr import *
from .vid48 import *
from .vimeo import *
from .vine import *
from .vk import *
from .w56 import *
from .xiami import *
from .yinyuetai import *
from .youku import *
from .youtube import *
from .ted import *
from .khan import *
